/* crlfetch.c - LDAP access
 *      Copyright (C) 2002 Klarlvdalens Datakonsult AB
 *
 * This file is part of DirMngr.
 *
 * DirMngr is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DirMngr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "crlfetch.h"
#include "dirmngr.h"
#include "../assuan/assuan.h"
#include <stdio.h>
#include <ldap.h>


/*! Fetch the value attr, put result in value, length in valuelen.
 *  Returns 0 on succes
 */
static int get_attr_from_result( LDAP* ld, LDAPMessage* msg, const char* attr, 
				char** value, size_t* valuelen )
{
  LDAPMessage* item = ldap_first_entry( ld, msg );  
  struct berval** values;
  if( !item ) return -1;
  values = ldap_get_values_len( ld, item, attr );
  if( values ) {    
    *valuelen = values[0]->bv_len;
    *value = xmalloc( *valuelen * sizeof( char ) );
    memcpy( *value, values[0]->bv_val, *valuelen );
    ldap_value_free_len( values );
    return ASSUAN_No_Error;
  } else return ASSUAN_No_CRL_For_Cert;
}

int crl_fetch( const char* issuer, char** value, size_t* valuelen )
{
  DIRMNGR_START_FNC;
  return attr_fetch_ldap( issuer, "certificateRevocationList;binary", value, valuelen );
}

int ca_cert_fetch( const char* dn, char** value, size_t* valuelen )
{
  DIRMNGR_START_FNC;
  return attr_fetch_ldap( dn, "cACertificate;binary", value, valuelen );
}

int attr_fetch_ldap(const char* dn,  const char* attr, char** value, size_t* valuelen )
{
  const char* host = dirmngr_opt.ldapserver;
  const int port = dirmngr_opt.ldapport;
  const char* user = dirmngr_opt.ldapuser;
  const char* pass = dirmngr_opt.ldappassword;
  char* attrs[] = {(char*)attr, NULL};
  int rc;

  LDAPMessage* result;
  LDAP* ld;

  dirmngr_debug( "trying to fetch attr %s from %s\n", attr, dn );

  ld = ldap_init( host, port );
  if( !ld ) {
    perror( "Error in ldap_init" );
    return ASSUAN_No_CRL_For_Cert;
  }
  if( ldap_simple_bind_s( ld, user, pass ) != LDAP_SUCCESS ) {
    ldap_perror( ld, "Error in ldap_simple_bind_s" );
    return ASSUAN_No_CRL_For_Cert;
  }
  
  if( ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "objectclass=*", attrs , 0, &result ) 
      != LDAP_SUCCESS ) {
    ldap_perror( ld, "Error during ldap_seach_s()" );
    return ASSUAN_No_CRL_For_Cert;
  }

  rc = get_attr_from_result( ld, result, attr, value, valuelen );
  if( rc ) {
    ldap_perror( ld, "Error during get_attr_from_result()" );
    return ASSUAN_No_CRL_For_Cert;
  }
  ldap_msgfree(result);
#if 0
  {
    FILE* file = fopen(attr, "w" );
    fwrite( *value, sizeof(char), *valuelen, file );
    fclose( file );
  }
#endif
  return ASSUAN_No_Error;
}
